@@ -72,7 +72,7 @@ function(x,nclass,n=NULL,CV=NULL,Ls=3,certain=NULL,alloc=list(q1=0.5,q2=0,q3=0.5
         
     
     # stratification
-    Nh <- Nc <- EYh <- EYc <- VYh <- nh <- nh.nonint <- opti <- TY <- se <- bias <- prop <- stratumID <- NULL
+    Nh <- Nc <- EYh <- EYc <- VYh <- nh <- nh.nonint <- opti.nh <- opti.nhnonint  <- TY <- se <- bias <- prop <- stratumID <- NULL
     out.get <- bh2nh(xgiven,N,bhfull,findn,n,CV,L,certain,q1,q2,q3,A,B,C,bias.penalty,takeall.adjust,rh,model,beta,sig2,ph,pcertain,gamma,epsilon)
     for(i in 1:length(out.get)) assign(names(out.get)[i],out.get[[i]])
     # Note : se est la racine carre de la variance de TY
@@ -81,7 +81,7 @@ function(x,nclass,n=NULL,CV=NULL,Ls=3,certain=NULL,alloc=list(q1=0.5,q2=0,q3=0.5
     # CV ou RRMSE est le relative root mean squared error de la moyenne ou de la somme : se/TY = (se/N)/(TY/N)
 
     # Sortie des rsultats
-    out<-list(Nh=Nh,nh=nh,n=sum(nh)+Nc,nh.nonint=nh.nonint,certain.info=c(Nc=Nc,meanc=EYc),opti.criteria=opti,bh=as.vector(bhfull[-c(1,L+1)]),
+    out<-list(Nh=Nh,nh=nh,n=sum(nh)+Nc,nh.nonint=nh.nonint,certain.info=c(Nc=Nc,meanc=EYc),opti.nh=opti.nh,opti.nhnonint=opti.nhnonint,bh=as.vector(bhfull[-c(1,L+1)]),
               meanh=ifelse(Nh==0,NA,EYh),varh=ifelse(Nh==0,NA,VYh),mean=TY/N,stderr=se/N,CV=se/TY,stratumID=stratumID,nclassh=nclassh[,nrgr],takeall=length(C),
               call=match.call(),date=date(),args=list(x=xgiven,nclass=nclass,n=n,CV=CV,Ls=Ls,certain=certain,alloc=alloc,rh=rh,model=model,model.control=model.control))
     class(out)<-"strata"
